@@ -28,7 +28,9 @@ class UnsupportedSerializedValueError(Exception):
 
 
 class CodeVersioningError(Exception):
-    pass
+    def __init__(self, message, bad_descriptor):
+        super(CodeVersioningError, self).__init__(message)
+        self.bad_descriptor = bad_descriptor
 
 
 class EntitySerializationError(Exception):
@@ -39,6 +41,10 @@ class EntityComputationError(Exception):
     pass
 
 
+class EntityValueError(ValueError):
+    pass
+
+
 class AttributeValidationError(Exception):
     pass
 
