@@ -3,7 +3,7 @@ import pytest
 import math
 import threading
 
-from ..helpers import RoundingProtocol, count_calls
+from ..helpers import RoundingProtocol
 from bionic.exception import AttributeValidationError, CodeVersioningError
 from bionic.protocols import PicklableProtocol
 
@@ -24,6 +24,13 @@ class ReadCountingProtocol(bn.protocols.PicklableProtocol):
         return super(ReadCountingProtocol, self).read(path)
 
 
+def raises_versioning_error_for_entity(entity_name):
+    return pytest.raises(
+        CodeVersioningError,
+        match=f".*function that outputs <{entity_name}>.*",
+    )
+
+
 # It would be nice to move the builder setup into fixtures, but since we need
 # to access the bound functions as well (to check the number of times they were
 # called), it's easiest to just have one long test.
@@ -37,7 +44,7 @@ def test_caching_and_invalidation(builder, make_counter, parallel_execution_enab
     xy_counter = make_counter()
 
     @builder
-    @count_calls(xy_counter)
+    @xy_counter
     def xy(x, y):
         return x * y
 
@@ -45,14 +52,14 @@ def test_caching_and_invalidation(builder, make_counter, parallel_execution_enab
 
     @builder
     @bn.persist(False)
-    @count_calls(yz_counter)
+    @yz_counter
     def yz(y, z):
         return y * z
 
     xy_plus_yz_counter = make_counter()
 
     @builder
-    @count_calls(xy_plus_yz_counter)
+    @xy_plus_yz_counter
     def xy_plus_yz(xy, yz):
         return xy + yz
 
@@ -385,7 +392,7 @@ def test_versioning_assist(builder, make_counter):
         call_counter.mark()
         return x * y
 
-    with pytest.raises(CodeVersioningError):
+    with raises_versioning_error_for_entity("f"):
         builder.build().get("f")
 
     builder.delete("f")
@@ -407,7 +414,7 @@ def test_versioning_assist(builder, make_counter):
         call_counter.mark()
         return y * x
 
-    with pytest.raises(CodeVersioningError):
+    with raises_versioning_error_for_entity("f"):
         builder.build().get("f")
 
     builder.delete("f")
@@ -427,7 +434,7 @@ def test_versioning_assist(builder, make_counter):
         call_counter.mark()
         return x ** y
 
-    with pytest.raises(CodeVersioningError):
+    with raises_versioning_error_for_entity("f"):
         builder.build().get("f")
 
     builder.delete("f")
@@ -469,7 +476,7 @@ def test_indirect_versioning_assist(builder, make_counter):
         y_call_counter.mark()
         return 4
 
-    with pytest.raises(CodeVersioningError):
+    with raises_versioning_error_for_entity("y"):
         builder.build().get("f")
 
     @builder  # noqa: F811
@@ -488,7 +495,7 @@ def test_indirect_versioning_assist(builder, make_counter):
         y_call_counter.mark()
         return len("xxxx")
 
-    with pytest.raises(CodeVersioningError):
+    with raises_versioning_error_for_entity("y"):
         builder.build().get("f")
 
     @builder  # noqa: F811
@@ -514,6 +521,73 @@ def test_indirect_versioning_assist(builder, make_counter):
     assert f_call_counter.times_called() == 0
 
 
+def test_indirect_nonpersisted_versioning_assist(builder, make_counter):
+    y_call_counter = make_counter()
+    f_call_counter = make_counter()
+
+    builder.set("core__versioning_mode", "assist")
+
+    builder.assign("x", 2)
+
+    @builder
+    @bn.persist(False)
+    def y():
+        y_call_counter.mark()
+        return 3
+
+    @builder
+    def f(x, y):
+        f_call_counter.mark()
+        return x + y
+
+    assert builder.build().get("f") == 5
+    assert y_call_counter.times_called() == 1
+    assert f_call_counter.times_called() == 1
+
+    @builder  # noqa: F811
+    @bn.persist(False)
+    def y():  # noqa: F811
+        y_call_counter.mark()
+        return 4
+
+    with raises_versioning_error_for_entity("y"):
+        builder.build().get("f")
+
+
+def test_indirect_nonpersisted_versioning_auto(builder, make_counter):
+    y_call_counter = make_counter()
+    f_call_counter = make_counter()
+
+    builder.set("core__versioning_mode", "auto")
+
+    builder.assign("x", 2)
+
+    @builder
+    @bn.persist(False)
+    def y():
+        y_call_counter.mark()
+        return 3
+
+    @builder
+    def f(x, y):
+        f_call_counter.mark()
+        return x + y
+
+    assert builder.build().get("f") == 5
+    assert y_call_counter.times_called() == 1
+    assert f_call_counter.times_called() == 1
+
+    @builder  # noqa: F811
+    @bn.persist(False)
+    def y():  # noqa: F811
+        y_call_counter.mark()
+        return 4
+
+    assert builder.build().get("f") == 6
+    assert y_call_counter.times_called() == 1
+    assert f_call_counter.times_called() == 1
+
+
 def test_versioning_auto(builder, make_counter):
     call_counter = make_counter()
 
@@ -678,7 +752,7 @@ def test_all_returned_results_are_deserialized(builder, make_counter):
 
     @builder
     @RoundingProtocol()
-    @count_calls(counter)
+    @counter
     def pi():
         return math.pi
 
@@ -731,21 +805,21 @@ def test_deps_not_called_when_values_not_changed(builder, make_counter):
     xy_counter = make_counter()
 
     @builder
-    @count_calls(xy_counter)
+    @xy_counter
     def xy(x, y):
         return x * y
 
     yz_counter = make_counter()
 
     @builder
-    @count_calls(yz_counter)
+    @yz_counter
     def yz(y, z):
         return y * z
 
     xy_plus_yz_counter = make_counter()
 
     @builder
-    @count_calls(xy_plus_yz_counter)
+    @xy_plus_yz_counter
     def xy_plus_yz(xy, yz):
         return xy + yz
 
@@ -777,7 +851,7 @@ def test_gather_cache_invalidation(builder, make_counter):
 
     @builder
     @bn.gather("x", "x", "df")
-    @count_calls(counter)
+    @counter
     def z(df, y):
         return df["x"].sum() + y
 
@@ -803,7 +877,7 @@ def test_gather_cache_invalidation_with_over_vars(builder, make_counter):
 
     @builder
     @bn.gather("x", "y", "df")
-    @count_calls(counter)
+    @counter
     def z(df):
         return df.sum().sum()
 
@@ -843,7 +917,7 @@ def test_complex_input_type(builder, make_counter):
     counter = make_counter()
 
     @builder
-    @count_calls(counter)
+    @counter
     def x_plus_y(x, y):
         return x + y
 
@@ -868,7 +942,7 @@ def test_persisting_none(builder, make_counter):
     counter = make_counter()
 
     @builder
-    @count_calls(counter)
+    @counter
     def none():
         return None
 
@@ -884,7 +958,7 @@ def test_disable_memory_caching(builder, make_counter):
     @builder
     @protocol
     @bn.memoize(False)
-    @count_calls(counter)
+    @counter
     def x():
         return 1
 
@@ -898,7 +972,7 @@ def test_disable_memory_caching(builder, make_counter):
     @protocol
     @bn.persist(False)
     @bn.memoize(False)
-    @count_calls(counter)
+    @counter
     def x():  # noqa: F811
         return 1
 
@@ -916,7 +990,7 @@ def test_disable_default_memory_caching(builder, make_counter):
 
     @builder
     @x_protocol
-    @count_calls(x_counter)
+    @x_counter
     def x():
         return 1
 
@@ -935,7 +1009,7 @@ def test_disable_default_memory_caching(builder, make_counter):
 
     @builder
     @y_protocol
-    @count_calls(y_counter)
+    @y_counter
     def y():
         return 2
 
@@ -953,7 +1027,7 @@ def test_disable_default_memory_caching(builder, make_counter):
     @builder
     @z_protocol
     @bn.memoize(True)
-    @count_calls(z_counter)
+    @z_counter
     def z():
         return 3
 
@@ -1010,7 +1084,7 @@ def test_changes_per_run_and_not_persist(
     @builder
     @bn.persist(False)
     @bn.changes_per_run
-    @count_calls(x_plus_one_counter)
+    @x_plus_one_counter
     def x_plus_one(x):
         return x + 1
 
@@ -1018,21 +1092,21 @@ def test_changes_per_run_and_not_persist(
 
     @builder
     @bn.persist(False)
-    @count_calls(x_plus_two_counter)
+    @x_plus_two_counter
     def x_plus_two(x_plus_one):
         return x_plus_one + 1
 
     x_plus_three_counter = make_counter()
 
     @builder
-    @count_calls(x_plus_three_counter)
+    @x_plus_three_counter
     def x_plus_three(x_plus_two):
         return x_plus_two + 1
 
     x_plus_four_counter = make_counter()
 
     @builder
-    @count_calls(x_plus_four_counter)
+    @x_plus_four_counter
     def x_plus_four(x_plus_three):
         return x_plus_three + 1
 
@@ -1075,14 +1149,14 @@ def test_changes_per_run_and_not_persist(
     assert x_plus_four_counter.times_called() == 0
 
 
-def test_changes_per_run_and_persist(builder, make_counter, parallel_execution_enabled):
+def test_changes_per_run_and_persist(builder, make_counter):
     builder.assign("x", 5)
 
     x_plus_one_counter = make_counter()
 
     @builder
     @bn.changes_per_run
-    @count_calls(x_plus_one_counter)
+    @x_plus_one_counter
     def x_plus_one(x):
         return x + 1
 
@@ -1090,14 +1164,14 @@ def test_changes_per_run_and_persist(builder, make_counter, parallel_execution_e
 
     @builder
     @bn.persist(False)
-    @count_calls(x_plus_two_counter)
+    @x_plus_two_counter
     def x_plus_two(x_plus_one):
         return x_plus_one + 1
 
     x_plus_three_counter = make_counter()
 
     @builder
-    @count_calls(x_plus_three_counter)
+    @x_plus_three_counter
     def x_plus_three(x_plus_two):
         return x_plus_two + 1
 
@@ -1116,18 +1190,11 @@ def test_changes_per_run_and_persist(builder, make_counter, parallel_execution_e
 
     flow = builder.build()
     assert flow.get("x_plus_three") == 8
-    # x_plus_one changes per run and should be recomputed between runs.
+    # x_plus_one changes per run, so it needs to be recomputed for this new flow
+    # instance.
     assert x_plus_one_counter.times_called() == 1
-    if parallel_execution_enabled:
-        # When executed in parallel, we don't compute every entity in
-        # the subprocess. Subprocess computes a non-persisted entity
-        # only if it is required to compute any other entity.
-        assert x_plus_two_counter.times_called() == 0
-    else:
-        # Since the value does not persist, x_plus_two is recomputed.
-        assert x_plus_two_counter.times_called() == 1
-    # Since value of x_plus_one didn't change even though it changes per run,
-    # x_plus_three is not computed.
+    # The other entities haven't changed and shouldn't need to be recomputed.
+    assert x_plus_two_counter.times_called() == 0
     assert x_plus_three_counter.times_called() == 0
 
 
@@ -1139,13 +1206,13 @@ def test_changes_per_run_and_not_memoize(builder, make_counter):
     @bn.memoize(False)
     @bn.persist(False)
     @bn.changes_per_run
-    @count_calls(counter)
+    @counter
     def x_plus_one(x):
         return x + 1
 
     with pytest.warns(
         UserWarning,
-        match="aren't configured to be memoized but are decorated with @changes_per_run",
+        match="isn't configured to be memoized but is decorated with @changes_per_run",
     ):
         flow = builder.build()
         assert flow.get("x_plus_one") == 6
@@ -1155,7 +1222,7 @@ def test_changes_per_run_and_not_memoize(builder, make_counter):
 
     with pytest.warns(
         UserWarning,
-        match="aren't configured to be memoized but are decorated with @changes_per_run",
+        match="isn't configured to be memoized but is decorated with @changes_per_run",
     ):
         flow = builder.build()
         assert flow.get("x_plus_one") == 6
@@ -1197,14 +1264,197 @@ def test_updating_cache_works_only_with_immediate(builder):
 
 
 def test_multiple_outputs_all_persisted_at_once(builder, make_counter):
-    call_counter = make_counter()
+    counter = make_counter()
 
     @builder
     @bn.outputs("x", "y")
-    @count_calls(call_counter)
+    @counter
     def x_y():
         return 1, 2
 
     assert builder.build().get("x") == 1
     assert builder.build().get("y") == 2
-    assert call_counter.times_called() == 1
+    assert counter.times_called() == 1
+
+
+def test_complex_tuple_outputs_all_persisted_at_once(builder, make_counter):
+    counter = make_counter()
+
+    @builder
+    @bn.returns("w, ((x, y), z)")
+    @counter
+    def w_x_y_z():
+        return (1, ((2, 3), 4))
+
+    assert builder.build().get("w") == 1
+    assert builder.build().get("x") == 2
+    assert builder.build().get("y") == 3
+    assert builder.build().get("z") == 4
+    assert counter.times_called() == 1
+
+
+def test_complex_flow_outputs_all_persisted_at_once(builder, make_counter):
+    builder.assign("a_minus_one", values=[2, 3, 4])
+
+    a_counter = make_counter()
+    ax_counter = make_counter()
+    a_something_x_counter = make_counter()
+
+    @builder
+    @a_counter
+    def a(a_minus_one):
+        return a_minus_one + 1
+
+    @builder
+    @bn.persist(False)
+    @ax_counter
+    def x():
+        return 2
+
+    @builder
+    @bn.returns("(a_plus_x, a_minus_x), a_times_x")
+    @a_something_x_counter
+    def a_something_x(a, x):
+        return ((a + x), (a - x)), (a * x)
+
+    assert builder.build().get("a_plus_x", collection=set) == {5, 6, 7}
+    assert builder.build().get("a_minus_x", collection=set) == {1, 2, 3}
+    assert builder.build().get("a_times_x", collection=set) == {6, 8, 10}
+    assert a_counter.times_called() == 3
+    assert a_something_x_counter.times_called() == 3
+
+
+def test_avoid_recomputing_nonpersisted_dep(builder, make_counter):
+    a_counter = make_counter()
+
+    @builder
+    @bn.persist(False)
+    @a_counter
+    def a():
+        return 1
+
+    @builder
+    def b(a):
+        return a + 1
+
+    assert builder.build().get("b") == 2
+    assert builder.build().get("b") == 2
+    assert a_counter.times_called() == 1
+
+
+def test_caching_dir_with_whitespaces(builder, make_counter, tmp_path):
+    builder.set("core__persistent_cache__flow_dir", str(tmp_path / "BN TEST DATA"))
+    counter = make_counter()
+
+    @builder
+    @counter
+    def one():
+        return 1
+
+    assert builder.build().get("one") == 1
+    assert builder.build().get("one") == 1
+    assert counter.times_called() == 1
+
+
+@pytest.fixture
+def make_tracked_class():
+    """
+    Creates a "tracked" class which keeps a count of how many of its instances are in
+    memory.
+    """
+
+    def _make_tracked_class():
+        # The Tracked class we're about to make is not picklable (since it's nested in
+        # this function), so we'll also need a custom protocol for it.
+        class TrackedProtocol(bn.protocols.BaseProtocol):
+            def get_fixed_file_extension(self):
+                return "tracked"
+
+            def write(self, value, path):
+                path.write_text(str(value.value))
+
+            def read(self, path):
+                return Tracked(int(path.read_text()))
+
+        class Tracked:
+            n_instances_in_memory = 0
+            protocol = TrackedProtocol()
+
+            def __init__(self, value):
+                self.__class__.n_instances_in_memory += 1
+                self.value = value
+
+            def __del__(self):
+                self.__class__.n_instances_in_memory -= 1
+
+        return Tracked
+
+    return _make_tracked_class
+
+
+def test_non_memoized_value_is_garbage_collected(builder, make_tracked_class):
+    Tracked = make_tracked_class()
+
+    @builder
+    @bn.memoize(False)
+    @Tracked.protocol
+    def tracked_x():
+        return Tracked(1)
+
+    @builder
+    def x_plus_one(tracked_x):
+        assert Tracked.n_instances_in_memory == 1
+        return tracked_x.value + 1
+
+    @builder
+    def x_plus_two(x_plus_one):
+        assert Tracked.n_instances_in_memory == 0
+        return x_plus_one + 1
+
+    assert builder.build().get("x_plus_two") == 3
+
+
+def test_non_memoized_values_are_garbage_collected(builder, make_tracked_class):
+    Tracked = make_tracked_class()
+
+    @builder
+    @bn.memoize(False)
+    @Tracked.protocol
+    @bn.outputs("tracked_x", "tracked_y")
+    def _():
+        return Tracked(1), Tracked(-1)
+
+    @builder
+    def x_plus_one(tracked_x):
+        assert Tracked.n_instances_in_memory == 1
+        return tracked_x.value + 1
+
+    @builder
+    def x_plus_two(x_plus_one):
+        assert Tracked.n_instances_in_memory == 0
+        return x_plus_one + 1
+
+    assert builder.build().get("x_plus_two") == 3
+
+
+def test_non_memoized_complex_tuples_are_garbage_collected(builder, make_tracked_class):
+    Tracked = make_tracked_class()
+
+    @builder
+    @bn.memoize(False)
+    @Tracked.protocol
+    @bn.returns("(tracked_x, tracked_y), tracked_z")
+    def _():
+        return (Tracked(1), Tracked(-1)), Tracked(0)
+
+    @builder
+    def x_plus_one(tracked_x):
+        assert Tracked.n_instances_in_memory == 1
+        return tracked_x.value + 1
+
+    @builder
+    def x_plus_two(x_plus_one):
+        assert Tracked.n_instances_in_memory == 0
+        return x_plus_one + 1
+
+    assert builder.build().get("x_plus_two") == 3
